; -----------------------------------------------------------------------------
; Star Castle - Atari 2600
; Copyright (C) Chris Walton (cd-w) 2012 - <cwalton@gmail.com>
; -----------------------------------------------------------------------------

  ; DASM Processor Type
  PROCESSOR 6502

  ; VCS Constants & Macros
  INCLUDE "vcs.h"
  INCLUDE "macro.h"

  ; Game Data
  INCLUDE "startitle.h"
  INCLUDE "starsprites.h"
  INCLUDE "starexplosion.h"
  INCLUDE "starmusic.h"

  ; RAM+ Base Addresses
  IFNCONST RAM_BASE_ADDRESS
RAM_BASE_ADDRESS = $1000
  ENDIF
  IFNCONST RAM_BASE_WRITE_ADDRESS
RAM_BASE_WRITE_ADDRESS = RAM_BASE_ADDRESS
  ENDIF
  IFNCONST RAM_BASE_READ_ADDRESS
RAM_BASE_READ_ADDRESS = RAM_BASE_ADDRESS+$100
  ENDIF

  ; Start Vertical Blank Macro
  MAC BEGIN_VBLANK
.WaitOverscan
  lda INTIM
  bne .WaitOverscan
  lda #%00001110
.SyncLoop
  sta WSYNC
  sta VSYNC
  lsr
  bne .SyncLoop
  lda #VBLANKDELAY
  sta TIM64T
  ENDM
  
  ; Start Frame Macro
  MAC BEGIN_FRAME
.WaitVblank
  lda INTIM
  bne .WaitVblank
  sta VBLANK
  sta WSYNC
  lda #FRAMEDELAY
  sta TIM64T
  ENDM
  
  ; Start Overscan Macro
  MAC BEGIN_OVERSCAN
.WaitFrame
  lda INTIM
  bne .WaitFrame
  lda #2
  sta WSYNC
  sta VBLANK
  lda #OVERDELAY
  sta TIM64T
  ENDM

; -----------------------------------------------------------------------------
; Game Constants
; -----------------------------------------------------------------------------

DEBUG               = 1     ; Enable Debugging (1 = Yes)
NO_ILLEGAL_OPCODES  = 0     ; Allow Illegal Opcodes
FAKESPIN            = 0     ; Fake Spinner (1 = Yes)
RAMSPINNER          = $B0   ; Stall Code Address

; NTSC Time Constants
VBLANKDELAY   = 52          ; 60Hz - (44*76)/64 ~ 52
FRAMEDELAY    = 214         ; 60Hz - (180*76)/64 ~ 214
OVERDELAY     = 42          ; 60Hz - (35*76)/64 ~ 42

; Hiscore Constants
KEYWAIT       = 20          ; Default Debounce Delay
KEYSHORT      = 10          ; Short Debounce Delay
RETRYOP       = 5           ; Retry eeprom operations
MAXSCORES     = 10          ; Scores In Table

; Game Constants
RANDOMSEED    = 15          ; Initial Random Number
EXPLODEFRAMES = 119
YRINGS        = 40
YOFFSET       = 25
YSTART        = (YRINGS + (YOFFSET * 2))  ; 90
YSPACE        = YRINGS + YOFFSET          ; 65
MIDDLEX       = 84
MIDDLEY       = 45
XXX           = (MIDDLEX - 20)            ; 64
YYY           = (MIDDLEY - 20)            ; 25
BULLETXMAX    = 50
BULLETYMAX    = 35

; -----------------------------------------------------------------------------
; Game Variables
; -----------------------------------------------------------------------------

  SEG.U   VARS
  ORG     $80

; Game Variables
CYCLE         DS.B  1       ; Frame Counter
RANDOM        DS.B  1       ; Random Seed

DEBOUNCE      DS.B  1       ; 7 = Fire, 6 = Fire State, 5 = Pause
                            ; 4 = B&W State, 1 = Joystick, 0 = Select

COLMODE       DS.B  1       ; 0 = NTSC, 1 = PAL60, 2 = B&W

WAVE          DS.B  1       ; 0-5 = Wave, 6-7 = Colours (0=NTSC, 1=PAL60, 2=B&W)
SHIPS         DS.B  1       ; 0-5 = Ships, 7 = Ship Explosion

SCORE         DS.B  3       ; 3 Digit Score (BCD)
INITIALS      DS.B  2       ; Initials (packed), 7 = 2600(1)/7800(0) Console

MODE          DS.B  1       ; 0-4 = Timer, 5-7 = Mode
                            ; 000 = Main Game
                            ; 001 = Get Ready
                            ; 010 = Score + Wave (on new wave)
                            ; 011 = Score + Ships (after death)
                            ; 100 = Game Over
                            ; 101 = Paused
                            ; 110 = ??
                            ; 111 = Explosion

; Sound Variables
SFX           DS.B  1       ; 0-4 = Sound Position, 5-7 = Sound Effect
SIRENCOUNT    DS.B  1       ; Siren Counter (0->63)
SIRENOFFSET   DS.B  1       ; Siren Offset  (0->15)
SF1           DS.B  1       ; Siren Frequency 1
SF2           DS.B  1       ; Siren Frequency 2

; Ship Variables
SHIPX_HI      DS.B  1       ; Ship Coordinates (16-bit)
SHIPY_HI      DS.B  1
SHIPX_LO      DS.B  1
SHIPY_LO      DS.B  1
SHIPDIR_HI    DS.B  1       ; Ship Direction & Explosion Frame
SHIPDIR_LO    DS.B  1
SHIPVX_HI     DS.B  1       ; Ship Velocity (16-bit)
SHIPVY_HI     DS.B  1
SHIPVX_LO     DS.B  1
SHIPVY_LO     DS.B  1
SHIPVDIR      DS.B  1       ; Ship Velocity Direction

; Gun Variables
GUNDIR        DS.B  1       ; Gun Direction
GUNCOUNT      DS.B  1       ; Gun Movement Delay
GUNCOUNT2     DS.B  1       ; Gun Firing Delay

; Bullet Variables
BULLETX_HI    DS.B  2       ; Bullet Coordinates (16-bit)
BULLETY_HI    DS.B  2
BULLETX_LO    DS.B  2
BULLETY_LO    DS.B  2
BULLETDIR     DS.B  2       ; Bullet Direction (0->31)
BULLETXD      DS.B  2       ; Bullet Distance
BULLETYD      DS.B  2
BCLEAR        DS.B  1       ; Sector to Clear & Score

; Projectile
PROJX_HI      DS.B  1       ; Projectile Coordinates (16-bit)
PROJY_HI      DS.B  1
PROJX_LO      DS.B  1
PROJY_LO      DS.B  1
PROJDIR       DS.B  1       ; Projectile Direction (0->31)
PROJFRAME     DS.B  1       ; Projectile Animation Frame (0->15)

; Flickersort Variables
MPOS          DS.B  1       ; Visible Mine
BPOS          DS.B  1       ; Visible Bullet

; Mine Variables
MPHASE        DS.B  1       ; Mine Phase
MINECOL       DS.B  3       ; Mine Colour Cache

; Ring Variables
SECTORS       DS.B  1       ; 0-5 = Sector Counter
BREACH        DS.B  1       ; Inner Ring Breach (Combine with SECTORS?)
RING0COUNT    DS.B  1       ; Ring Rotation Delay Counters
RING1COUNT    DS.B  1
RING2COUNT    DS.B  1
OFFSET0       DS.B  12      ; Ring Coordinate Offsets
OFFSET1       DS.B  12
OFFSET2       DS.B  12
RINGCOL0      DS.B  1       ; Ring Colours
RINGCOL1      DS.B  1
RINGCOL2      DS.B  1

; Kernel Variables (TEMP)
PPTR          DS.B  3       ; Player Sprite
PMASKPTR      DS.B  3       ; Player Mask
BPTR          DS.B  2       ; Ball Pointer
MPTR          DS.B  2       ; Missile Pointer
LINE0         DS.B  2       ; Ring Pointers
LINE1         DS.B  2
LINE2         DS.B  2
LINE3         DS.B  2
LINE4         DS.B  2

; Temp Variables
TEMP          DS.B  1
TEMP2         DS.B  1

; STACK         DS.B  2       ; JSR Stack

; Game Temps
POS           EQU TEMP
COUNT         EQU TEMP2
ROW           EQU TEMP2
EXPLODE       EQU GUNCOUNT  ; Explosion Counter
SHIPANGLE     EQU PPTR+2    ; Ship Angle From Centre
MINEANGLE     EQU PMASKPTR+2      ; Mine Angle From Center/Ship
CURRSECTOR    EQU PPTR+1    ; Current Sector
; DISTANCE_HI   EQU BPTR+1    ; Square Distance From Centre
DISTANCE_LO   EQU BPTR      ; Square Distance From Centre
TEMPX         EQU MPTR
TEMPY         EQU MPTR+1
TEMPBX        EQU PMASKPTR
TEMPBY        EQU PMASKPTR+1
TEMPBPOS      EQU PMASKPTR+2
TEMPMAP       EQU PPTR+2

; Explosion Pointers
EXP0          EQU PPTR
EXP1          EQU PPTR+2
EXP2          EQU PMASKPTR+1
EXP3          EQU BPTR
EXP4          EQU MPTR

; Text Pointers
NPTR          EQU PPTR
LPTR          EQU PMASKPTR
PTR0          EQU MPTR
PTR1          EQU LINE0
PTR2          EQU LINE1
PTR3          EQU LINE2
PTR4          EQU LINE3
PTR5          EQU LINE4

; Title Screen Variables
SLOWCYCLE     EQU MODE
LOGOFG        EQU RINGCOL0
LOGOBG        EQU RINGCOL1
SPTR          EQU PPTR      ; 2   Sound Effects Table Pointer
TPTR          EQU PPTR      ; 2   Text Pointer
MESSAGE       EQU MPOS
MTIMER        EQU BPOS

; Title Music Variables
MEASURE       EQU SHIPX_HI
BEAT          EQU SHIPY_HI
TEMPOCOUNT    EQU SHIPX_LO
CHAN          EQU SHIPY_LO
TEMP16L       EQU SHIPDIR_HI
TEMP16H       EQU SHIPDIR_LO
FREQ          EQU SHIPVX_HI
ATTEN         EQU SHIPVY_HI

; High Score Table Variables
SLPTR         EQU LINE0     ; 2
SLLPTR        EQU LINE1     ; 2
SLHPTR        EQU LINE2     ; 2
HDPTR         EQU LINE3     ; 2
NAMECOL       EQU MINECOL   ; 3
HPOS          EQU PROJX_HI  ; 1
HROW          EQU PROJY_HI  ; 1

; HiScore Table Variables
HBUFF         EQU OFFSET0   ; (37)
HTEXT0        EQU HBUFF+0   ; HiScore Text
HTEXT1        EQU HBUFF+1
HTEXT2        EQU HBUFF+5
HTEXT3        EQU HBUFF+6
HTEXT4        EQU HBUFF+10
HTEXT5        EQU HBUFF+11
HTEXT6        EQU HBUFF+15
HTEXT7        EQU HBUFF+16
HTEXT8        EQU HBUFF+20
HTEXT9        EQU HBUFF+21
HTEXT10       EQU HBUFF+25
HTEXT11       EQU HBUFF+26
HTEXT12       EQU HBUFF+30
HTEXT13       EQU HBUFF+31

; Name Entry Variables
HPTR0         EQU HBUFF+0   ; 2
HPTR1         EQU HBUFF+2   ; 2
HPTR2         EQU HBUFF+4   ; 2
HPTR3         EQU HBUFF+6   ; 2
HPTR4         EQU HBUFF+8   ; 2
HPTR5         EQU HBUFF+10  ; 2
NAME          EQU HBUFF+31  ; 3
CURSOR        EQU HBUFF+34  ; 1
HSCORE        EQU HBUFF+35  ; 3

  ; Display Remaining RAM
  echo "----",($100 - *) , "bytes left (RAM)"

  SEG.U RAM_REGISTERS_WRITE
  ORG RAM_BASE_WRITE_ADDRESS

  ; RAM+ Write Variables
RAM_W
HI_WRITE
  ; Mines
MINEX_LO_W    DS.B  3
MINEX_HI_W    DS.B  3
MINEY_LO_W    DS.B  3
MINEY_HI_W    DS.B  3
MINECOUNT_W   DS.B  3
MINEDIR_W     DS.B  3       ; 0->4 = Direction/Sector, 5-7 = Mode
                            ; 000 = Unspawned
                            ; 001 = Ring 0
                            ; 010 = Ring 1
                            ; 011 = Ring 2
                            ; 100 = Explode
                            ; 111 = Homing
  ; Sector Map
SECTORMAP0_W  DS.B  12
SECTORMAP1_W  DS.B  12
SECTORMAP2_W  DS.B  12
  ; Grid
GRID_W        DS.B  200     ; 40 * 5
COLMODE_W     DS.B  1
OPERATION_W   DS.B  1

  SEG.U RAM_REGISTERS_READ
  ORG RAM_BASE_READ_ADDRESS

  ; RAM+ Read Variables
RAM_R
HI_READ
  ; Mines
MINEX_LO_R    DS.B  3
MINEX_HI_R    DS.B  3
MINEY_LO_R    DS.B  3
MINEY_HI_R    DS.B  3
MINECOUNT_R   DS.B  3
MINEDIR_R     DS.B  3       ; 0->4 = Direction/Sector, 5-7 = Mode
  ; Sector Map
SECTORMAP0_R  DS.B  12
SECTORMAP1_R  DS.B  12
SECTORMAP2_R  DS.B  12
  ; Grid
GRID_R        DS.B  200     ; 40 * 5
COLMODE_R     DS.B  1
ERRORCODE_R   DS.B  1

  ; Display Remaining RAM
  echo "----",((RAM_BASE_READ_ADDRESS + 256) - *) , "bytes left (RAM+)"

; -----------------------------------------------------------------------------
; BANKS 0-5
; -----------------------------------------------------------------------------

; BANK 0 FFF5 - GAME LOGIC
; BANK 1 FFF6 - COLLISION DETECTION
; BANK 2 FFF7 - GAME KERNEL
; BANK 3 FFF8 - RING ROTATION
; BANK 4 FFF9 - EXPLOSIONS & GUN FRAMES
; BANK 5 FFFA - TITLE & MUSIC
; BANK 6 FFFB - STARTUP & HISCORES

  INCLUDE bank0.asm
  INCLUDE bank1.asm
  INCLUDE bank2.asm
  INCLUDE bank3.asm
  INCLUDE bank4.asm
  INCLUDE bank5.asm
  INCLUDE bank6.asm

; -----------------------------------------------------------------------------

